<template>
  <div class="layout-columns-aside">
    <el-scrollbar>
      <ul @mouseleave="onColumnsAsideMenuMouseleave()">
        <!-- {{ state.columnsAsideList }} -->
        <li v-for="(v, k) in state.columnsAsideList" :key="k" @click="onColumnsAsideMenuClick(v, k)"
          @mouseenter="onColumnsAsideMenuMouseenter(v, k)"
          :class="{ 'layout-columns-active': state.liIndex === k, 'layout-columns-hover': state.liHoverIndex === k }"
          :title="$t(v.meta.title)" :ref="(el) => {
            if (el) columnsAsideOffsetTopRefs[k] = el;
          }
            ">
          <!-- 不显示超链接或者显示iframe -->
          <div :class="themeConfig.columnsAsideLayout" v-if="!v.meta.isLink || (v.meta.isLink && v.meta.isIframe)">
            <SvgIcon :name="v.meta.icon" />
            <div class="columns-vertical-title font12">
              {{ $t(v.meta.title) && $t(v.meta.title).length >= 4
                ? $t(v.meta.title).substr(0, themeConfig.columnsAsideLayout === 'colunmns-vertical' ? 4 : 3)
                : $t(v.meta.title)
              }}
            </div>
          </div>
          <!-- 否则a新窗口打开 -->
          <div :class="themeConfig.columnsAsideLayout" v-else>
            <a :href="v.meta.isLink" target="_blank">
              <SvgIcon :name="v.meta.icon" />
              <div class="columns-vertical-title font12">
                {{ $t(v.meta.title) && $t(v.meta.title).length >= 4
                  ? $t(v.meta.title).substr(0, themeConfig.columnsAsideLayout === 'colunmns-vertical' ? 4 : 3)
                  : $t(v.meta.title)
                }}
              </div>
            </a>
          </div>
        </li>
        <!-- 分栏高亮风格themeConfig.columnsAsideStyle: 'columns-round', -->
        <div ref="columnsAsideActiveRef" :class="themeConfig.columnsAsideStyle"></div>
      </ul>
    </el-scrollbar>
  </div>
</template>

<script setup lang="ts">
import { storeToRefs } from 'pinia';
import { ref, reactive, onMounted, nextTick, watch, onUnmounted } from 'vue'

import { useRoutesList } from '/@/stores/routesList'
import { useThemeConfig } from '/@/stores/themeConfig'
import mittBus from '/@/utils/mitt';

import { useRoute, useRouter, onBeforeRouteUpdate, RouteRecordRaw } from 'vue-router'
import { ElScrollbar } from 'element-plus';
import pinia from '/@/stores';

// 定义变量内容
const columnsAsideOffsetTopRefs = ref<RefType>([])//这边需要[] 返回多个数组，切记！！！
const state = reactive<ColumnsAsideState>({
  columnsAsideList: [],
  liIndex: 0,
  liOldIndex: null,
  liHoverIndex: null,
  liOldPath: null,
  difference: 0,
  routeSplit: [],
})
const stores = useRoutesList()//找store状态管理
const { routesList, isColumnsMenuHover, isColumnsNavHover } = storeToRefs(stores)

const storesThemeConfig = useThemeConfig()
const { themeConfig } = storeToRefs(storesThemeConfig)

const route = useRoute()//找原生路由对象
const router = useRouter()

const columnsAsideActiveRef = ref()
// 设置/过滤路由（非静态路由/是否显示在菜单中）-->获取state.columnsAsideList
const setFilterRoutes = () => {
  state.columnsAsideList = filterRoutesFun(routesList.value)
  const resData: MittMenu = setSendChildren(route.path);
  if (Object.keys(resData).length <= 0) return false;
  onColumnsAsideDown(resData.item?.k)// 设置高亮动态位置
  mittBus.emit('setSendColumnsChildren', resData)
}
// 路由过滤递归函数
const filterRoutesFun = <T extends RouteItem>(arr: T[]): T[] => {
  return arr.filter((item: T) => !item.meta?.isHide).map((item: T) => {
    item = Object.assign({}, item);
    if (item.children) item.children = filterRoutesFun(item.children)
    return item;
  })
}
// 传送当前子级数据到菜单中--从一级传递到这边子级
const setSendChildren = (path: string) => {
  // debugger
  const currentPathSplit = path.split('/')
  let currentData: MittMenu = { children: [] };
  state.columnsAsideList.map((v: RouteItem, k: number) => {
    if (v.path === `/${currentPathSplit[1]}`) {
      v['k'] = k
      currentData['item'] = { ...v }
      currentData['children'] = [{ ...v }]
      if (v.children) currentData['children'] = v.children
    }
  });
  return currentData;
}

// 设置高亮动态位置
const onColumnsAsideDown = (k: number) => {
  nextTick(() => {
    setColumnsAsideMove(k);// 设置菜单高亮位置移动
  })
}
// 设置菜单高亮位置移动
const setColumnsAsideMove = (k: number) => {
  state.liIndex = k
  columnsAsideActiveRef.value.style.top = `${columnsAsideOffsetTopRefs.value[k].offsetTop + state.difference}px`; //高亮定位浮动
}
// 菜单高亮点击事件
const onColumnsAsideMenuClick = (v: RouteItem, k: number) => {
  setColumnsAsideMove(k)
  let { path, redirect } = v
  if (redirect) router.push(redirect)
  else router.push(path)
}
//鼠标移入时，显示当前的子级菜单--一级显示二级，三级...--->@mouseenter-->鼠标移入
const onColumnsAsideMenuMouseenter = (v: RouteRecordRaw, k: number) => {
  if (!themeConfig.value.isColumnsMenuHoverPreload) return false
  let { path } = v
  state.liOldPath = path;
  state.liOldIndex = k
  state.liHoverIndex = k
  mittBus.emit('setSendColumnsChildren', setSendChildren(path));
  stores.setColumnsMenuHover(false);
  stores.setColumnsNavHover(true);
}

// 鼠标移走时，显示原来的子级菜单----->@mouseleave-->鼠标移走
const onColumnsAsideMenuMouseleave = async () => {
  await stores.setColumnsNavHover(false)
  // 添加延时器，防止拿到的 store.state.routesList 值不是最新的
  setTimeout(() => {
    if (!isColumnsMenuHover && !isColumnsNavHover) mittBus.emit('restoreDefault')//'发送' --第一步
  }, 100);
}

// 页面加载时
onMounted(() => {
  setFilterRoutes();// 设置/过滤路由（非静态路由/是否显示在菜单中）
  // 销毁变量，防止鼠标再次移入时，保留了上次的记录
  mittBus.on('restoreDefault', () => { state.liOldIndex = null; state.liOldPath = null })//onMounted的生命周期'接收'--第二步
})
// 页面卸载时
onUnmounted(() => {
  mittBus.off('restoreDefault', () => { })//页面卸载时候，销毁重置监听--第三步
})
// 路由更新时--tagsview标签切换路由变更，最左边菜单也要跟着高亮定位--核心--切记（onBeforeRouteUpdate路由切换生命周期）
onBeforeRouteUpdate((to) => {//入口555--tagsview右边切换，左边这边核心位置--监听触发
  setColumnsMenuHighlight(to.path)
  mittBus.emit('setSendColumnsChildren', setSendChildren(to.path))//在layout-->component-->aside.vue(onBeforeMount页面加载前监听)--setSendColumnsChildren最左边一级菜单=》第一步’路由更新-发送‘
})
// tagsView 点击时，根据路由查找下标 columnsAsideList，实现左侧菜单高亮
const setColumnsMenuHighlight = (path: string) => {
  state.routeSplit = path.split('/')
  state.routeSplit.shift()//删除前面，['/','home']==>shift() 返回 home
  const routeFirst = `/${state.routeSplit[0]}`
  const currentSplitRoute = state.columnsAsideList.find((v: RouteItem) => v.path === routeFirst)
  if (!currentSplitRoute) return false;
  // 延迟拿值，防止取不到
  setTimeout(() => {
    onColumnsAsideDown(currentSplitRoute.k)//k为当前路由item下面的index
  }, 0);
}

// 监听布局配置信息的变化，动态增加菜单高亮位置移动像素--核心显示，监听显示column二级菜单
watch(
  pinia.state,
  (val) => {
    // debugger
    val.themeConfig.themeConfig.columnsAsideStyle === 'columnsRound' ? (state.difference = 3) : (state.difference = 0);
    if (!val.routesList.isColumnsMenuHover && !val.routesList.isColumnsNavHover) {
      state.liHoverIndex = null;
      mittBus.emit('setSendColumnsChildren', setSendChildren(route.path));
    } else {
      state.liHoverIndex = state.liOldIndex;
      if (!state.liOldPath) return false;
      mittBus.emit('setSendColumnsChildren', setSendChildren(state.liOldPath));//watch监听路由选项，显示二级菜单以及高亮效果
    }
  },
  {
    deep: true,
  }
);
</script>
<style scoped lang="scss">
.layout-columns-aside {
  width: 70px;
  height: 100%;
  background: var(--next-bg-columnsMenuBar);

  ul {
    position: relative;

    .layout-columns-active {
      color: var(--next-bg-columnsMenuBarColor) !important;
      transition: 0.3s ease-in-out;
    }

    .layout-columns-hover {
      color: var(--el-color-primary);

      a {
        color: var(--el-color-primary);
      }
    }

    li {
      color: var(--next-bg-columnsMenuBarColor);
      width: 100%;
      height: 50px;
      text-align: center;
      display: flex;
      cursor: pointer;
      position: relative;
      z-index: 1;

      &:hover {
        @extend .layout-columns-hover;
      }

      .columns-vertical {
        margin: auto;

        .columns-vertical-title {
          padding-top: 1px;
        }
      }

      .columns-horizontal {
        display: flex;
        height: 50px;
        width: 100%;
        align-items: center;
        padding: 0 5px;

        i {
          margin-right: 3px;
        }

        a {
          display: flex;

          .columns-horizontal-title {
            padding-top: 1px;
          }
        }
      }

      a {
        text-decoration: none;
        color: var(--next-bg-columnsMenuBarColor);
      }
    }

    .columns-round {
      //最左边菜单蓝色背景样式
      background: var(--el-color-primary);
      color: var(--el-color-white);
      position: absolute;
      left: 50%;
      top: 2px;
      height: 44px;
      width: 65px;
      transform: translateX(-50%); //居中布局
      z-index: 0;
      transition: 0.3s ease-in-out;
      border-radius: 5px;
    }

    .columns-card {
      @extend .columns-round;
      top: 0;
      height: 50px;
      width: 100%;
      border-radius: 0;
    }
  }
}
</style>
